@@ -36,7 +36,6 @@
 #include <errno.h>
 #include <string.h>
 
-#include <sys/ioctl.h>
 #include <sys/stat.h>
 
 #include <glib.h>
@@ -57,6 +56,11 @@ typedef struct _CdCache {
 
 #ifdef HAVE_HAL
   LibHalContext *ctx;
+  gboolean has_medium;
+#endif
+#ifdef HAVE_HAL_0_5
+  /* If the disc is a media, have the UDI available here */
+  char *disc_udi;
 #endif
 
   /* capabilities of the device */
@@ -157,7 +161,8 @@ cd_cache_get_dev_from_volumes (GnomeVFSVolumeMonitor *mon, const char *device,
     volume = list->data;
     if (!(pdev = gnome_vfs_volume_get_device_path (volume)))
       continue;
-    if (!(pdev2 = get_device (pdev, NULL))) {
+    pdev2 = get_device (pdev, NULL);
+    if (!pdev2) {
       g_free (pdev);
       continue;
     }
@@ -189,6 +194,7 @@ cd_cache_get_dev_from_volumes (GnomeVFSVolumeMonitor *mon, const char *device,
 
   return found;
 }
+
 static gboolean
 cd_cache_get_dev_from_drives (GnomeVFSVolumeMonitor *mon, const char *device,
 			      char **mountpoint, GnomeVFSDrive **d)
@@ -206,7 +212,8 @@ cd_cache_get_dev_from_drives (GnomeVFSVolumeMonitor *mon, const char *device,
     drive = list->data;
     if (!(pdev = gnome_vfs_drive_get_device_path (drive)))
       continue;
-    if (!(pdev2 = get_device (pdev, NULL))) {
+    pdev2 = get_device (pdev, NULL);
+    if (!pdev2) {
       g_free (pdev);
       continue;
     }
@@ -218,7 +225,7 @@ cd_cache_get_dev_from_drives (GnomeVFSVolumeMonitor *mon, const char *device,
       mnt = gnome_vfs_drive_get_activation_uri (drive);
       if (mnt && strncmp (mnt, "file://", 7) == 0) {
         *mountpoint = g_strdup (mnt + 7);
-      } else /*if (mnt && strncmp (mnt, "cdda://", 7) == 0)*/ {
+      } else {
 	*mountpoint = NULL;
       }
       found = TRUE;
@@ -260,9 +267,17 @@ cd_cache_new_hal_ctx (void)
     return NULL;
 
   dbus_error_init (&error);
+#if (DBUS_VERSION_MAJOR == 0) && (DBUS_VERSION_MINOR < 60)
   conn = dbus_bus_get (DBUS_BUS_SYSTEM, &error);
+  g_warning ("might crash, see http://bugzilla.gnome.org/show_bug.cgi?id=345203");
+#else
+  conn = dbus_bus_get_private (DBUS_BUS_SYSTEM, &error);
+#endif
   if (conn != NULL && !dbus_error_is_set (&error)) {
-    libhal_ctx_set_dbus_connection (ctx, conn);
+    if (!libhal_ctx_set_dbus_connection (ctx, conn)) {
+      libhal_ctx_free (ctx);
+      return NULL;
+    }
     return ctx;
   }
 
@@ -272,7 +287,9 @@ cd_cache_new_hal_ctx (void)
   }
 
   libhal_ctx_shutdown (ctx, NULL);
-  libhal_ctx_free(ctx);
+  libhal_ctx_free (ctx);
+  if (conn != NULL)
+    dbus_connection_unref (conn);
 
   return NULL;
 }
@@ -399,11 +416,13 @@ cd_cache_has_medium (CdCache *cache)
       return FALSE;
     }
     retval = TRUE;
+    cache->disc_udi = udi;
+  } else {
+    g_free (udi);
   }
 
   if (devices != NULL)
     libhal_free_string_array (devices);
-  g_free (udi);
 
   return retval;
 }
@@ -440,7 +459,7 @@ cd_cache_open_device (CdCache *cache,
 		      GError **error)
 {
   /* not a medium? */
-  if (cache->is_media == FALSE) {
+  if (cache->is_media == FALSE || cache->has_medium != FALSE) {
     return TRUE;
   }
 
@@ -449,6 +468,7 @@ cd_cache_open_device (CdCache *cache,
 	_("Please check that a disc is present in the drive."));
     return FALSE;
   }
+  cache->has_medium = TRUE;
 
   return TRUE;
 }
@@ -546,8 +566,14 @@ cd_cache_free (CdCache *cache)
 #ifdef HAVE_HAL
   if (cache->ctx != NULL) {
 #ifdef HAVE_HAL_0_5
+    DBusConnection *conn;
+
+    conn = libhal_ctx_get_dbus_connection (cache->ctx);
     libhal_ctx_shutdown (cache->ctx, NULL);
     libhal_ctx_free(cache->ctx);
+    dbus_connection_close (conn);
+
+    g_free (cache->disc_udi);
 #elif HAVE_HAL_0_2
     hal_shutdown (cache->ctx);
 #endif
@@ -566,27 +592,51 @@ static MediaType
 cd_cache_disc_is_cdda (CdCache *cache,
 		       GError **error)
 {
-  MediaType type = MEDIA_TYPE_DATA;
-  GList *vol, *item;
-
   /* We can't have audio CDs on disc, yet */
   if (cache->is_media == FALSE)
-    return type;
+    return MEDIA_TYPE_DATA;
+  if (!cd_cache_open_device (cache, error))
+    return MEDIA_TYPE_ERROR;
 
-  for (vol = item = gnome_vfs_drive_get_mounted_volumes (cache->drive);
-      item != NULL; item = item->next) {
-    char *mnt = gnome_vfs_volume_get_activation_uri (item->data);
-    if (mnt && strncmp (mnt, "cdda://", 7) == 0) {
-      g_free (mnt);
-      type = MEDIA_TYPE_CDDA;
-      break;
+#ifdef HAVE_HAL_0_5
+  {
+    DBusError error;
+    dbus_bool_t is_cdda;
+
+    dbus_error_init (&error);
+
+    is_cdda = libhal_device_get_property_bool (cache->ctx,
+	cache->disc_udi, "volume.disc.has_audio", &error);
+
+    if (dbus_error_is_set (&error)) {
+      g_warning ("Error checking whether the volume is an audio CD: %s",
+	  error.message);
+      dbus_error_free (&error);
+      return MEDIA_TYPE_ERROR;
     }
-    g_free (mnt);
+    return is_cdda ? MEDIA_TYPE_CDDA : MEDIA_TYPE_DATA;
+  }
+#else
+  {
+    MediaType type = MEDIA_TYPE_DATA;
+    GList *vol, *item;
+
+    for (vol = item = gnome_vfs_drive_get_mounted_volumes (cache->drive);
+	item != NULL; item = item->next) {
+      char *mnt = gnome_vfs_volume_get_activation_uri (item->data);
+      if (mnt && strncmp (mnt, "cdda://", 7) == 0) {
+	g_free (mnt);
+	type = MEDIA_TYPE_CDDA;
+	break;
+      }
+      g_free (mnt);
+    }
+    g_list_foreach (vol, (GFunc) gnome_vfs_volume_unref, NULL);
+    g_list_free (vol);
   }
-  g_list_foreach (vol, (GFunc) gnome_vfs_volume_unref, NULL);
-  g_list_free (vol);
 
   return type;
+#endif
 }
 
 static gboolean
@@ -649,6 +699,36 @@ cd_cache_disc_is_vcd (CdCache *cache,
     return MEDIA_TYPE_ERROR;
   if (!cache->mountpoint)
     return MEDIA_TYPE_ERROR;
+#ifdef HAVE_HAL_0_5
+  if (cache->is_media != FALSE) {
+    DBusError error;
+    dbus_bool_t is_vcd;
+
+    dbus_error_init (&error);
+
+    is_vcd = libhal_device_get_property_bool (cache->ctx,
+	cache->disc_udi, "volume.disc.is_vcd", &error);
+
+    if (dbus_error_is_set (&error)) {
+      g_warning ("Error checking whether the volume is a VCD: %s",
+	  error.message);
+      dbus_error_free (&error);
+      return MEDIA_TYPE_ERROR;
+    }
+    if (is_vcd != FALSE)
+      return MEDIA_TYPE_VCD;
+    is_vcd = libhal_device_get_property_bool (cache->ctx,
+	cache->disc_udi, "volume.disc.is_svcd", &error);
+
+    if (dbus_error_is_set (&error)) {
+      g_warning ("Error checking whether the volume is an SVCD: %s",
+	  error.message);
+      dbus_error_free (&error);
+      return MEDIA_TYPE_ERROR;
+    }
+    return is_vcd ? MEDIA_TYPE_VCD : MEDIA_TYPE_DATA;
+  }
+#endif
   /* first is VCD, second is SVCD */
   if (cd_cache_file_exists (cache, "MPEGAV", "AVSEQ01.DAT") ||
       cd_cache_file_exists (cache, "MPEG2", "AVSEQ01.MPG"))
@@ -668,6 +748,25 @@ cd_cache_disc_is_dvd (CdCache *cache,
     return MEDIA_TYPE_ERROR;
   if (!cache->mountpoint)
     return MEDIA_TYPE_ERROR;
+#ifdef HAVE_HAL_0_5
+  if (cache->is_media != FALSE) {
+    DBusError error;
+    dbus_bool_t is_dvd;
+
+    dbus_error_init (&error);
+
+    is_dvd = libhal_device_get_property_bool (cache->ctx,
+	cache->disc_udi, "volume.disc.is_videodvd", &error);
+
+    if (dbus_error_is_set (&error)) {
+      g_warning ("Error checking whether the volume is a DVD: %s",
+	  error.message);
+      dbus_error_free (&error);
+      return MEDIA_TYPE_ERROR;
+    }
+    return is_dvd ? MEDIA_TYPE_DVD : MEDIA_TYPE_DATA;
+  }
+#endif
   if (cd_cache_file_exists (cache, "VIDEO_TS", "VIDEO_TS.IFO"))
     return MEDIA_TYPE_DVD;
 
@@ -739,8 +838,10 @@ totem_cd_detect_type_with_url (const char *device,
     return MEDIA_TYPE_ERROR;
 
   type = cd_cache_disc_is_cdda (cache, error);
-  if (type == MEDIA_TYPE_ERROR && *error != NULL)
+  if (type == MEDIA_TYPE_ERROR && *error != NULL) {
+    cd_cache_free (cache);
     return type;
+  }
 
   if ((type == MEDIA_TYPE_DATA || type == MEDIA_TYPE_ERROR) &&
       (type = cd_cache_disc_is_vcd (cache, error)) == MEDIA_TYPE_DATA &&
