@@ -106,21 +106,28 @@ totem_sidebar_is_visible (Totem *totem)
 }
 
 void
-totem_sidebar_setup (Totem *totem)
+totem_sidebar_setup (Totem *totem, gboolean visible, const char *page_id)
 {
 	GtkWidget *item, *item2;
-	gboolean visible;
 
 	item = glade_xml_get_widget (totem->xml, "tmw_main_pane");
 	totem->sidebar = ev_sidebar_new ();
 	ev_sidebar_add_page (EV_SIDEBAR (totem->sidebar),
 			"playlist", _("Playlist"),
 			GTK_WIDGET (totem->playlist));
+	ev_sidebar_add_page (EV_SIDEBAR (totem->sidebar),
+			"properties", _("Properties"),
+			GTK_WIDGET (totem->properties));
+	if (page_id != NULL) {
+		ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar),
+				page_id);
+	} else {
+		ev_sidebar_set_current_page (EV_SIDEBAR (totem->sidebar),
+				"playlist");
+	}
+	gtk_widget_set_sensitive (GTK_WIDGET (totem->properties), FALSE);
 	gtk_paned_pack2 (GTK_PANED (item), totem->sidebar, FALSE, FALSE);
 
-	visible = gconf_client_get_bool (totem->gc,
-				GCONF_PREFIX"/sidebar_shown",
-				NULL);
 	totem->sidebar_shown = visible;
 
 	item = glade_xml_get_widget (totem->xml, "tmw_sidebar_button");
@@ -138,10 +145,14 @@ totem_sidebar_setup (Totem *totem)
 			G_CALLBACK (totem_sidebar_toggle), totem);
 
 	gtk_widget_show_all (totem->sidebar);
-	
 	gtk_widget_realize (totem->sidebar);
-	
+
 	if (!visible)
 		gtk_widget_hide (totem->sidebar);
 }
 
+const char *
+totem_sidebar_get_current_page (Totem *totem)
+{
+	return ev_sidebar_get_current_page (EV_SIDEBAR (totem->sidebar));
+}
